// 先准备fields, 钻取的行列fields, 画布的高度与宽度,
import * as _ from "lodash";

export class EchartsGridOptions {
    fields: string[];
    columns: string[];
    rows: string[];
    dataset: any[];
    cellWidth: number;
    cellHeight: number;
    canvasWidth: number;
    canvasHeight: number;
    seriesOptions: any;
}

export function init(options: EchartsGridOptions) {
    const getDatum = (fields: string[], keys: { [key: number]: any }, start: number) => {
        const fieldIndex = options.fields.findIndex(field => field === fields[start++]);
        const datum: {
            keys: string[],
            map: Map<string, any>,
            level?: number,
            dataset: any[],
        } = {
            keys: [],
            map: new Map<string, any>(),
            level: 1,
            dataset: []
        };
        const dataset = options.dataset.filter(d => {
            let match: boolean = true;
            if (keys) {
                for (let key of _.keys(keys)) {
                    match = d[key] === keys[key];
                    if (!match)
                        break;
                }
            }
            if (match) {
                if (datum.keys.indexOf(d[fieldIndex]) < 0) {
                    datum.keys.push(d[fieldIndex]);
                }
            }
            return match;
        });
        if (start < fields.length) {
            for (let key of datum.keys) {
                let addKey: any = {};
                addKey[fieldIndex] = key;
                datum.map[key] = this.getDatum(fields, {...keys, ...addKey}, start);
            }
            if (datum.keys[0]) {
                datum.level += datum.map[datum.keys[0]].level;
            }
        } else {
            // 这是最后一个了， 返回数据集
            datum.dataset = dataset;
        }
        return datum;
    };

    const columns = getDatum(options.columns.slice(0, options.columns.length - 2), null, 0);
    const rows = getDatum(options.rows.slice(0, options.rows.length - 2), null, 0);
    const rowsMeta: {
        height: number;
        dataset: any[];
    }[] = [];
    const columnsMeta: {
        width: number;
        dataset: any[];
    }[] = [];

    const columnsHeight = (columns.level + 1) * options.cellHeight;
    const rowsWidth = (rows.level + 1) * options.cellWidth;
    const graphics: any[] = [];
    const grids: any[] = [];
    const series: any[] = [];
    const xAxis: any[] = [];
    const yAxis: any[] = [];

    const xField = options.rows.length > 0 ? options.rows[options.rows.length - 1] : null;
    const yField = options.columns.length > 0 ? options.columns[options.columns.length - 1] : null;
    //利用echarts的graph画列头与行头
    const canvasWidth = options.canvasWidth;
    const canvasHeight = options.canvasHeight;
    const height = options.cellHeight;
    const width = options.cellWidth;

    function createCharts(xField: string, yField: string, xAxis: any[], yAxis: any[], graphics: any[], grids: any[], series: any[], offsetX: number, offsetY: number) {
        let startY = 0;
        let startX = 0;
        let gridIndex = 0;
        columnsMeta.forEach((c, columnIndex) => {
            startY = 0;
            rowsMeta.forEach((r, rowIndex) => {
                const gridOption: any = {
                    x: offsetX + startX,
                    y: offsetY + startY + 10,
                    width: c.width - 10,
                    height: r.height - 20,
                    borderWidth: 0
                };

                const showXAxis = rowIndex == rowsMeta.length - 1;
                if (showXAxis) {
                    gridOption.height += 10;
                }
                xAxis.push({
                    type: 'category',
                    gridIndex,
                    show: showXAxis,
                    data: c.dataset.map(d => d[xField])
                });

                const showYAxis = columnIndex == 0;
                yAxis.push({type: 'value', gridIndex, axisLabel: {show: showYAxis}, axisTick: {show: showYAxis}});
                grids.push(gridOption);
                graphics.push({
                    type: 'rect',
                    left: offsetX + startX,
                    top: offsetY + startY,
                    z: 100,
                    shape: {
                        width: c.width,
                        height: r.height
                    },
                    style: {
                        fill: 'transparent',
                        stroke: "#000"
                    }
                });
                series.push({
                    xAxisIndex: gridIndex,
                    yAxisIndex: gridIndex,
                    data: r.dataset.map(d => d[yField]),
                    ...options.seriesOptions
                });
                gridIndex++;
                startY += r.height;
            });
            startX += c.width;
        });
    }

    function createColumnHeader(graphics: any[], width: number, height: number, text: string, offsetX: number) {
        graphics.push({
            type: 'group',
            left: offsetX,
            top: 0,
            width,
            height,
            children: [
                {
                    type: 'rect',
                    left: '0',
                    top: 0,
                    shape: {width, height},
                    style: {fill: "transparent"}
                },
                {type: 'text', left: 'center', top: 'center', style: {fill: '#000', text}}
            ]
        });
    }

    function createXHeader(graphics: any[], offsetX: number, offsetY: number, height: number) {
        let left = 0;
        console.log(offsetY);
        rowsMeta.forEach(r => {
            offsetY += r.height;
        });
        columnsMeta.forEach((c) => {
            graphics.push({
                type: 'group',
                left: offsetX + left,
                top: offsetY + 2,
                width: c.width,
                height: height,
                children: [
                    {
                        type: 'rect',
                        left: 0,
                        top: 0,
                        shape: {width: c.width, height: height},
                        style: {fill: "transparent"}
                    },
                    {
                        type: 'line',
                        left: c.width,
                        top: 0,
                        shape: {y1: 0, y2: height},
                        style: {fill: "#000"}
                    },
                    {
                        type: 'line',
                        left: 0,
                        top: 0,
                        shape: {x1: 0, x2: c.width},
                        style: {fill: "#000"}
                    }
                ]
            });
            left += c.width;
        });
    }


    function createRowsHeader(fields: string[], graphics: any[], width: number, height: number, cellWidth: number, offsetY: number) {
        const cellHeight = height - offsetY;
        const childGraphic: any = {
            type: 'group', left: 2, top: offsetY + 2, width, cellHeight,
            children: []
        };
        fields.forEach((text, i) => {
            childGraphic.children.push({
                type: 'group',
                left: i * cellWidth,
                top: 0,
                width: cellWidth,
                height: cellHeight,
                children: [
                    {
                        type: 'rect',
                        left: 0,
                        top: 0,
                        shape: {width: cellWidth, height: cellHeight},
                        style: {fill: "transparent"}
                    },
                    {
                        type: 'line',
                        left: 0,
                        top: cellHeight,
                        shape: {x1: 0, x2: cellWidth},
                        style: {fill: "#000"}
                    },
                    {type: 'text', left: 'center', top: 'center', style: {fill: '#000', text}}
                ]
            });
        });
        childGraphic.children.push({
            type: 'group',
            left: fields.length * cellWidth,
            top: 0,
            width: cellWidth,
            height: cellHeight,
            children: [
                {
                    type: 'rect',
                    left: 0,
                    top: 0,
                    shape: {width: cellWidth, height: cellHeight},
                    style: {fill: "transparent"}
                },
                {
                    type: 'line',
                    left: 0,
                    top: cellHeight,
                    shape: {x1: 0, x2: cellWidth},
                    style: {fill: "#000"}
                }
            ]
        });
        graphics.push(childGraphic);
    }

    function createColumnDimensions(columns: { keys: string[], map: Map<string, any>, dataset: any[] }, graphics: any[], level: number, width: number, height: number, offsetX: number, offsetY: number) {
        const columnWidth = width / columns.keys.length;
        const nextLevel = level + 1;
        columns.keys.forEach((key, i) => {
            const childGraphic: any = {
                type: 'group', left: 0, top: nextLevel * height, width: columnWidth, height,
                children: []
            };
            graphics.push({
                type: 'group',
                left: offsetX + i * columnWidth,
                top: offsetY + level * height,
                width: columnWidth,
                height,
                children: [
                    {
                        type: 'line',
                        left: columnWidth,
                        top: 0,
                        shape: {x1: 0, y1: 0, x2: 0, y2: height},
                        style: {stroke: '#000'}
                    },
                    {type: 'text', left: 'center', top: 'center', style: {fill: '#000', text: key}},
                    childGraphic
                ]
            });

            if (columns.map[key]) {
                this.createColumnDimensions(columns.map[key] as any, childGraphic.children, nextLevel, columnWidth, height, offsetX, offsetY);
            } else {
                graphics[graphics.length - 1].children.push({
                    type: 'line',
                    left: '0',
                    top: height,
                    shape: {x1: 0, y1: 0, x2: columnWidth, y2: 0},
                    style: {stroke: '#000'}
                });
                //维度集
                columnsMeta.push({
                    width: columnWidth,
                    dataset: columns.dataset
                });
            }
        });
    }

    function createRowDimensions(columns: { keys: string[], map: Map<string, any>, dataset: any[] }, graphics: any[], level: number, height: number, width: number, offsetX: number, offsetY: number) {
        const columnHeight = height / columns.keys.length;
        const nextLevel = level + 1;
        columns.keys.forEach((key, i) => {
            const childGraphic: any = {
                type: 'group', left: nextLevel * width, top: 0, width, height: columnHeight,
                children: []
            };
            graphics.push({
                type: 'group', left: level * width, top: offsetY + i * columnHeight, width, height: columnHeight,
                children: [
                    {
                        type: 'line',
                        left: '0',
                        top: columnHeight,
                        shape: {x1: 0, y1: 0, x2: width, y2: 0},
                        style: {stroke: '#000'}
                    },
                    {type: 'text', left: 'center', top: 'center', style: {fill: '#000', text: key}},
                    childGraphic
                ]
            });

            if (columns.map[key]) {
                this.createRowDimensions(columns.map[key] as any, childGraphic.children, nextLevel, columnHeight, width, offsetX, offsetY);
            } else {
                console.log(nextLevel * width);
                graphics.push({
                    type: 'group',
                    left: nextLevel * width,
                    top: offsetY + i * columnHeight,
                    width,
                    height: columnHeight,
                    children: [
                        {
                            type: 'rect',
                            left: 0,
                            top: 0,
                            shape: {width, height: columnHeight},
                            style: {fill: 'transparent'}
                        },
                        {
                            type: 'line',
                            left: width,
                            top: 0,
                            shape: {x1: 0, y1: 0, x2: 0, y2: columnHeight},
                            style: {stroke: '#000'}
                        },
                        {
                            type: 'line',
                            left: 0,
                            top: columnHeight,
                            shape: {x1: 0, y1: 0, x2: width, y2: 0},
                            style: {stroke: '#000'}
                        }
                    ]
                });
                //维度集
                rowsMeta.push({
                    height: columnHeight,
                    dataset: columns.dataset
                });
            }
        });
    }


    createColumnHeader(graphics, canvasWidth - rowsWidth, height, options.columns.splice(0, options.columns.length - 2).join(" / "), rowsWidth);
    createColumnDimensions(columns, graphics, 0, canvasWidth - rowsWidth - width, height, rowsWidth + width, height);

    createRowsHeader(options.rows.splice(0, options.rows.length - 2), graphics, rowsWidth, columnsHeight, width, columnsHeight - height);
    createRowDimensions(rows, graphics, 0, canvasHeight - columnsHeight - height, width, 0, columnsHeight);
    //创建底部空间放刻度尺
    createXHeader(graphics, rowsWidth + width + 2, columnsHeight, height);
    createCharts(xField, yField, xAxis, yAxis, graphics, grids, series, rowsWidth + width + 2, columnsHeight + 2);

    return {
        grids,
        series,
        graphics,
        yAxis,
        xAxis
    }
}
